@@ -1,4 +1,4 @@
-from .version import __version__
+__version__ = '0.13.0'
 
 def can_process(input_file):
     '''
@@ -13,6 +13,12 @@ def can_process(input_file):
         return False
 
 def open_bilingualfile(bilingualfile):
+    '''
+    Opens a compatible xliff variant.
+
+    Args:
+        bilingualfile: Path to a .kxliff, .xliff, or .sdlxliff file.
+    '''
     try:
         from .kxliff import KXLIFF
         return KXLIFF.open_bilingualfile(bilingualfile)
